{#
## This file is part of Zenodo.
## Copyright (C) 2016 CERN.
##
## Zenodo is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Zenodo is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
##
## In applying this licence, CERN does not waive the privileges and immunities
## granted to it by virtue of its status as an Intergovernmental Organization
## or submit itself to any jurisdiction.
#}

{% macro render_field(field, icon="", placeholder='', autofocus=False) %}
  <div class="form-group {% if icon %} has-feedback{% endif %}{% if field.errors %} has-error{% endif %}">
    {%- set extras = dict(autofocus="") if autofocus else dict() %}
    {{field(class_="form-control", placeholder=placeholder or _(field.label.text), **extras)}}
    {%- if icon %}
    <i class="{{icon}} form-control-feedback" aria-hidden="true" ></i>
    {%- endif%}
    {%- if field.description %}
    <small class="text-muted">{{field.description}}</small>
    {%- endif %}
  </div>
{% endmacro %}
