# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Access request signals."""

from __future__ import absolute_import, print_function

from blinker import Namespace

_signals = Namespace()

request_accepted = _signals.signal('request-accepted')

request_rejected = _signals.signal('request-rejected')

request_created = _signals.signal('request-created')

request_confirmed = _signals.signal('request-confirmed')

link_created = _signals.signal('link-created')

link_revoked = _signals.signal('link-revoked')
